from datetime import date

def welcome_email_template(recipient, app_title: str, verification_link) -> str:

    verify_button = f"""
    <a href="{verification_link}" target="_blank">
        <div class="button">
            <p>Verify your email address</p>
        </div>
    </a>
    """

    return f"""
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Welcome to Our App</title>
    <style>
        body {{
            font-family: Arial, sans-serif;
            line-height: 1.6;
            color: #333333;
            margin: 0;
            padding: 0;
            background-color: #f4f4f4;
        }}
        .container {{
            max-width: 600px;
            margin: 0 auto;
            padding: 20px;
            background-color: #ffffff;
        }}
        .header {{
            background-color: #3498db;
            color: #ffffff;
            padding: 20px;
            text-align: center;
        }}
        .content {{
            padding: 20px;
            background-color: #ffffff;
        }}
        .footer {{
            background-color: #34495e;
            color: #ffffff;
            padding: 10px 20px;
            text-align: center;
            font-size: 12px;
        }}
        .button {{
            display: inline-block;
            padding: 10px 20px;
            background-color: #2ecc71;
            color: #ffffff;
            text-decoration: none;
            border-radius: 5px;
            margin-top: 20px;
        }}
    </style>
</head>
<body>
    <div class="container">
        <div class="header">
            <h1>Welcome to {app_title}!</h1>
        </div>
        <div class="content">
            <p>Hi {recipient.name},</p>
            <p>Thank you for registering with {app_title}. We're excited to have you on board!</p>
                {verify_button if not recipient.email_verified else ''}
            <p>If you have any questions or need assistance, please don't hesitate to reach out to our support team.</p>
        </div>
        <div class="footer">
            <p>&copy; {date.today().year} {app_title}. All rights reserved.</p>
        </div>
    </div>
</body>
</html>
    """



import json
from pydantic import BaseModel


class DeepLink(BaseModel):
    ff_page: str
    deep_link_parameter_name: str | None = None
    destination_id: str | int | None = None

    @property
    def ff_route(self) -> dict | None:
        if self.ff_page and self.deep_link_parameter_name and self.destination_id:
            return {
                "initialPageName": self.ff_page,
                "parameterData": json.dumps({
                    self.deep_link_parameter_name: self.destination_id
                })
             }
        elif self.ff_page:
            return {
                "initialPageName": self.ff_page
            }


class UserNotificationsRequest(BaseModel):
    recipient_ids: list[str] | str
    title: str
    body: str
    deeplink_page_name: str | None = None
    deep_link_parameter_name: str | None = None
    destination_id: str | int | None = None


class Notification(BaseModel):
    title: str
    body: str
    image_url: str | None = None
    deep_link: DeepLink | None = None


class UserNotificationCreate(BaseModel):
    user_id: str
    notification: Notification
    is_read: bool = False



from fastapi import Request, status
from fastapi.exceptions import RequestValidationError
from starlette.responses import JSONResponse
from mosayic.logger import get_logger

logger = get_logger(__name__)


async def handle_uncaught_exceptions(request: Request, call_next: callable):
    """Send a message to Slack when an uncaught exception occurs"""
    try:
        response = await call_next(request)
    except Exception as e:
        logger.exception(str(e))
        response = JSONResponse(status_code=status.HTTP_500_INTERNAL_SERVER_ERROR, content={"detail": str(e)})
    return response


async def handle_validation_error(request: Request, exc: RequestValidationError):
    """Exception handler to get more information about 422 errors. Not technically invoked as middleware, but acts as such."""
    exc_str = f'{exc}'.replace('\n', ' ').replace('   ', ' ')
    logger.error(f"{request}: {exc_str}")
    content = {'status_code': 422, 'message': exc_str, 'data': None}
    return JSONResponse(content=content, status_code=422)



import resend
from mosayic.logger import get_logger
from mosayic.services.email.email_templates import welcome_email_template

logger = get_logger(__name__)


class ResendService:

    def __init__(self, resend_api_key):
        resend.api_key = resend_api_key
        self.params = resend.Emails.SendParams()

    # async def send_welcome_email(self, recipient, sender, verification_link: str | None) -> dict:
    #     logger.info("Sending welcome recipient_email to %s...",  recipient.email)
    #     self.params['to'] = recipient.email
    #     self.params['from'] = f"{sender_name } <{self.settings.from_email}>"
    #     self.params['subject'] = f'Welcome to {self.settings.app_title}'
    #     self.params['html'] = welcome_email_template(recipient, self.settings, verification_link)
    #     if self.params['to'] and self.params['from'] and self.params['subject'] and self.params['html']:
    #         response = resend.Emails.send(self.params)
    #         logger.info("The welcome email has been sent to: %s",  recipient.email)
    #         return response
    #     logger.error("Email not sent. Missing parameters: %s", self.params)





########### Firebase auth routes ##############

# @router.get("/admin/auth/users", response_model=list[FirebaseAuthUser])
# async def get_users(users: list = Depends(get_users_list)):
#     """
#     Get a list of all Firebase users. This route is only accessible to admins.
#     """
#     # TODO users pagination
#     return users


# @router.get("/admin/auth/users/{user_uid}", response_model=FirebaseAuthUser)
# async def get_user_by_id(users: list = Depends(get_firebase_user_by_uid)):
#     """
#     Get a Firebase user by their UID. This route is only accessible to admins.
#     """
#     return users


# @router.post("/admin/auth/delete-user/{user_uid}", dependencies=[Depends(delete_user)])
# async def admin_user_delete() -> None:
#     pass



from fastapi import APIRouter, Depends, status
# from mosayic.services.notifications.notification_service import ping_notification_status, send_notification_to_users, mark_notification_as_read
from mosayic.logger import get_logger

logger = get_logger(__name__)

notifications_router = APIRouter(
    prefix='/notifications',
)


# @notifications_router.get('/status', status_code=status.HTTP_200_OK, response_model=dict)
# async def notification_status(status = Depends(ping_notification_status)):
#     """Get the notification status. Returns a json object showing the count of unread notifications"""
#     return status


# @notifications_router.post('/{pk}/mark-as-read', status_code=status.HTTP_200_OK, dependencies=[Depends(mark_notification_as_read)])
# async def mark_as_read() -> None:
#     """Mark a notification as read"""
#     pass


# @notifications_router.post('/send', status_code=status.HTTP_201_CREATED, dependencies=[Depends(send_notification_to_users)])
# async def send_notification() -> None:
#     """Send a notification to users. Admins may pass 'all' for the field 'recipient_ids' to send to all users."""
#     pass



import importlib.resources as resources
from fastapi import APIRouter, Request, status
from fastapi.templating import Jinja2Templates
from mosayic.logger import get_logger
# from mosayic.database.supabase.supabase_functions import get_request
# from mosayic.constants import TERMS_AND_CONDITIONS_ROW_ID, PRIVACY_POLICY_ROW_ID, COMPLIANCE_TABLE
from mosayic.services.email.resend_service import ResendService

templates_dir = resources.files("mosayic") / "webpages/templates"
templates = Jinja2Templates(directory=str(templates_dir))

logger = get_logger(__name__)

webpages_router = APIRouter(
    prefix='/webpages',
)

# @webpages_router.get('/terms-and-conditions', status_code=status.HTTP_200_OK)
# async def get_terms_and_conditions(request: Request):
#     """Reads the app_compliance table in Supabase, and returns the terms and conditions HTML"""
#     data = await get_request(COMPLIANCE_TABLE, eq=('id', TERMS_AND_CONDITIONS_ROW_ID))
#     if len(data) != 1:
#         raise ValueError("Terms and conditions not found or wrong number of rows returned")
#     return templates.TemplateResponse(
#         request=request,
#         name="layout.html",
#         context={"html_content": data[0].get('html')},
#     )

# @webpages_router.get('/privacy-policy', status_code=status.HTTP_200_OK)
# async def get_privacy_policy(request: Request):
#     """Reads the app_compliance table in Supabase, and returns the privacy policy HTML"""
#     data = await get_request(COMPLIANCE_TABLE, eq=('id', PRIVACY_POLICY_ROW_ID))
#     if len(data) != 1:
#         raise ValueError("Privacy policy not found or wrong number of rows returned")
#     return templates.TemplateResponse(
#         request=request,
#         name="layout.html",
#         context={"html_content": data[0].get('html')},
#     )


@webpages_router.get('/data-removal-request', status_code=status.HTTP_200_OK)
async def get_data_deletion_request_form(request: Request, app_title):
    """Returns the data deletion request web form"""
    return templates.TemplateResponse(
        request=request,
        name="data_deletion_request_form.html",
        context={"app_title": app_title},
    )


@webpages_router.post('/data-removal-request', status_code=status.HTTP_200_OK)
async def get_data_deletion_request_submit(request: Request):
    form_data = await request.form()
    html = f"""
        <p>Dear Admin,</p>
        <p>A user has requested to delete their data. Please take the necessary steps to delete their data.</p>
        <p>Details:</p>
        <p>Name: {form_data.get('name')}</p>
        <p>Email: {form_data.get('email')}</p>
        <br>
        <p>Message:</p>
        <p>{form_data.get('message')}</p>
        <br>

        <p>This is an automated email.</p>
    """
    resend_service = ResendService()
    # await resend_service.send_email_to_admins(
    #     subject='Data deletion request',
    #     html=html
    # )
    return templates.TemplateResponse(
        request=request, name="data_deletion_request_submitted.html"
    )


@webpages_router.get('/support', status_code=status.HTTP_200_OK)
async def get_support_page(request: Request, app_title: str, support_email: str):
    return templates.TemplateResponse(
        request=request,
        name="support_page.html"
    )
